%%%%%%%%%% ACTIVE CONTOURS CHAN-VESE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% Fchanvese.m
%%%%%%%%%% By G. Galiano (08/04/2013)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% INPUT
% I:					image
% nu:					external force parameter (non-negative)
% tol:					stopping criterium for relative error 
% sc:                   size of blocks for initial datum
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% OUTPUT
% u:		level set function
% I1:		image corresponding to original I, which may have been resized
% num_iter:	number of iterations used till convergence or stopping 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% FUNCTIONS
% preprocessing
% initial_datum
% kappa
% plot_level
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 function [u I num_iter]= Fchanvese(I_ori,nu,tol,sc,steps_redistancing)
 if nargin==4;steps_redistancing=30;end
 if nargin==3;steps_redistancing=30;sc=2;end
 if nargin==2;steps_redistancing=30;sc=2;tol=1.e-5;end
 if nargin==1;steps_redistancing=30;sc=2;tol=1.e-5;nu=1;end
 if nargin==0;steps_redistancing=30;sc=2;tol=1.e-5;nu=1;
     I_ori = imread('cameraman.tif');end
 %%%%%%%%%% PRE-PROCESSING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
I=preprocessing(I_ori);
%%%%%%%%%% INITIAL DATUM FOR LEVEL SET, u\in (0,1) %%%%%%%%%%%%%%%%%%%%%%%%
u=initial_datum(I,sc);
%%%%%%%%%% DISPLAY SETTINGS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure();
subplot(2,2,1); imshow(I); title('Input Image');
subplot(2,2,2); contour(flipud(u), [0 0], 'r','LineWidth',1);
title('initial contour');
subplot(2,2,3); title('Segmentation');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% FINITE DIFFERENCES INIZIALIZATION
n=size(I,1)*size(I,2);
num_iter_max=1000;
u_old=zeros(size(u));
dt=0.5;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% MAIN LOOP
for i=1:num_iter_max
	inidx = find(u>=0); % frontground index
	outidx = find(u<0); % background index
    c1 = sum(sum(I.*Heaviside(u)))/(length(inidx)+eps);		%Av. inside u 
	c2 = sum(sum(I.*(1-Heaviside(u))))/(length(outidx)+eps);% Av.outside u 
	force_image=-(I-c1).^2+(I-c2).^2;					% internal force 
	force =nu*kappa(u)./max(max(abs(kappa(u))))+force_image;% external force
	u = u+dt*force;
	% STOPPING CRITERIUM
    invariant_sum=sum(sum((u<0)==(u_old<0)));
	ER=(n-invariant_sum)/n;
	if ER<tol & i>10 	% For some cases ER<tol is satisfied for too small i
		disp('Convergence achieved');
        subplot(2,2,4); imshow(u<=0); title('Segmentation');drawnow;
  		num_iter=i;
  		break;
    end
    u_old=u;
    % Matlab Euclidian distance transform AND GRAPHIC OUTPUT
	if(mod(i,steps_redistancing) == 0)
        u0=(u>0);%Binary image
        u = bwdist(u0)-bwdist(1-u0)+im2double(u0)-.5;
		plot_level(I,u,i);
		disp(['Relative error = ',num2str(ER)]);
	end;
end;
num_iter=i;
subplot(2,2,4); imshow(u<=0); title('Segmentation');
% figure
% subplot(2,1,1);plot_level(I,u)
% title('Zero level set')
% subplot(2,1,2);imshow(u<=0);;
% title('Zero level set')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function I=preprocessing(I);
%%%%% Converting to gray-scale
if size(I,3)== 3;I = rgb2gray(uint8(I));I=double(I);
elseif size(I,3) == 2;I = mean(double(I),3);I=double(I);
else;I = double(I);
end
%%%%% Resizing on base 256x256
s = 256./min(size(I,1),size(I,2)); 
if s<1;I = imresize(I,s);end
%%%%% Rescaling into [0,1]
m = min(min(I));M = max(max(I));
d=1/(M-m);
if d>eps;I = d*(I-m);end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function u2=initial_datum(I,sc)
% Initial datum defined as blocks of zeros and ones
block=[zeros(2^sc,2^sc) ones(2^sc,2^sc); ones(2^sc,2^sc) zeros(2^sc,2^sc)];
s=ceil(size(I)/2^(sc+1));
u1=repmat(block,s(1),s(2));
u1=u1(1:size(I,1),1:size(I,2));
u0=(u1>0);						% Binary image
u2 = bwdist(u0)-bwdist(1-u0)+im2double(u0)-.5;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function H=Heaviside(z)
% Heaviside step function (smoothed version)
epsilon=10^(-5);
H=zeros(size(z,1),size(z,2));
idx1=find(z>epsilon);
idx2=find(z<epsilon & z>-epsilon);
H(idx1)=1;
for i=1:length(idx2)
    H(idx2(i))=1/2*(1+z(idx2(i))/epsilon+1/pi*sin(pi*z(idx2(i))/epsilon));
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function KG = kappa(I)
u=I;%double(I);
[ux,uy]=gradient(u);
G=sqrt(ux.^2+uy.^2);
K=divergence(ux./(G+eps),uy./(G+eps));
KG = K.*G;
KG(1,:) = eps;KG(end,:) = eps;
KG(:,1) = eps;KG(:,end) = eps;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_level(I, u ,i)
% Show zero level set of function u over image I
% Optionally, pass an integer argument, i, for number of iterations  
imshow(I,'initialmagnification','fit','displayrange',[0 1]);
hold on;
contour(u, [0 0], 'r','LineWidth',1);
hold off; 
if(nargin==3)
	title([num2str(i) ' Iterations']);
else
	title('Zero level set');
end 
drawnow;

